# -*- mode: python; python-indent-offset: 4; indent-tabs-mode: nil -*-
# SPDX-License-Indentifier: MPL-2.0

"""SenseHat adapter for Mozilla WebThings Gateway."""

from sense_hat import SenseHat
from gateway_addon import Adapter
from .sense_hat_device import SenseHatDevice
from .sense_hat_imu_device import SenseHatImuDevice
from .sense_hat_light_device import SenseHatLightDevice


class SenseHatAdapter(Adapter):
    """Adapter for Sense Hat"""

    def __init__(self, verbose=False):
        """
        Initialize the object.

        verbose -- whether or not to enable verbose logging
        """

        self.pairing = False
        self.addon_name = 'sense-hat'
        self.DEBUG = True
        self.name = self.__class__.__name__
        self.URL = 'https://github.com/rzr/sense-hat-webthing'
        Adapter.__init__(self,
                         self.addon_name, self.addon_name, verbose=verbose)

        try:
            self.controller = SenseHat()
            device = SenseHatDevice(self)
            self.handle_device_added(device)
            if self.DEBUG:
                print("sense_hat_device created")
            self.devices[device.id].connected = True
            self.devices[device.id].connected_notify(True)

            imu_device = SenseHatImuDevice(self)
            self.handle_device_added(imu_device)
            if self.DEBUG:
                print("sense_hat_imu_device created")
            self.devices[imu_device.id].connected = True
            self.devices[imu_device.id].connected_notify(True)

            lightDevice = SenseHatLightDevice(self)
            self.handle_device_added(lightDevice)
            if self.DEBUG:
                print("sense_hat_light_device created")
            self.devices[lightDevice.id].connected = True
            self.devices[lightDevice.id].connected_notify(True)

        except Exception as ex:
            print("error: Could not create sense_hat_device: " + str(ex))



# -*- mode: python; python-indent-offset: 4; indent-tabs-mode: nil -*-
# SPDX-License-Indentifier: MPL-2.0

"""SenseHat adapter for Mozilla WebThings Gateway."""

from sense_hat import SenseHat
from gateway_addon import Device, Property
import threading
import time

_POLL_INTERVAL = 5

class SenseHatDevice(Device):
    """SenseHat device type."""

    def __init__(self, adapter):
        """
        Initialize the object.

        adapter -- the Adapter managing this device
        """

        Device.__init__(self, adapter, 'sense-hat')

        self._id = 'sense-hat'
        self.id = 'sense-hat'
        self.adapter = adapter
        self.controller = adapter.controller

        self.name = 'SenseHat'
        self.description = 'Expose SenseHat sensors'
        self.links = [
            {
                'rel': 'alternate',
                'mediaType': 'text/html',
                'href': adapter.URL
            }
        ]
        self._type = ['TemperatureSensor']
        try:
            self.properties['humidity'] = SenseHatProperty(
                self,
                "humidity",
                {
                    '@type': 'NumberProperty',
                    'label': "Humidity",
                    'type': 'integer',
                    'unit': '%',
                    'readOnly': True
                },
                0)
            self.properties['pressure'] = SenseHatProperty(
                self,
                "pressure",
                {
                    '@type': 'NumberProperty',
                    'label': "Pressure",
                    'readOnly': True,
                    'type': 'number',
                    'unit': 'hPa'
                },
                0)
            self.properties['temperature'] = SenseHatProperty(
                self,
                'temperature',
                {
                    '@type': 'TemperatureProperty',
                    'label': "Temperature",
                    'type': 'number',
                    'unit': 'degrees celsius',
                    'readOnly': True
                },
                0)
            self.properties['down'] = SenseHatProperty(
                self,
                'down',
                {
                    '@type': 'PushedProperty',
                    'label': "Down",
                    'type': 'boolean',
                    'readOnly': True
                },
                False)
            self.properties['left'] = SenseHatProperty(
                self,
                'left',
                {
                    '@type': 'PushedProperty',
                    'label': "Left",
                    'type': 'boolean',
                    'readOnly': True
                },
                False)
            self.properties['right'] = SenseHatProperty(
                self,
                'right',
                {
                    '@type': 'PushedProperty',
                    'label': "Right",
                    'type': 'boolean',
                    'readOnly': True
                },
                False)
            self.properties['up'] = SenseHatProperty(
                self,
                'up',
                {
                    '@type': 'PushedProperty',
                    'label': "Up",
                    'type': 'boolean',
                    'readOnly': True
                },
                False)
            t = threading.Thread(target=self.poll)
            t.daemon = True
            t.start()

            events = threading.Thread(target=self.handle_events)
            events.daemon = True
            events.start()

            self.pairing = True
            print("info: Adapter started")

        except Exception as ex:
            print("error: Adding properties: " + str(ex))

    def poll(self):
        """Poll the device for changes."""
        while True:
            time.sleep(_POLL_INTERVAL)
            try:
                for prop in self.properties.values():
                    prop.update()
            except Exception as ex:
                print("error: Polling properties: " + str(ex))
                print(prop.name)
                continue

    def handle_events(self):
        while True:
             event = self.controller.stick.wait_for_event(True)
             prop = self.properties[event.direction]
             value = True if (str(event.action) == 'held') else False
             prop.set_cached_value_and_notify(value)

class SenseHatProperty(Property):

    def __init__(self, device, name, description, value):
        Property.__init__(self, device, name, description)
        self.device = device
        self.title = name
        self.name = name
        self.description = description
        self.value = value
        self.set_cached_value(value)

    def update(self):
        if self.name == 'humidity':
            value = self.device.controller.humidity
        elif self.name == 'pressure':
            value = self.device.controller.get_pressure()
        elif self.name == 'temperature':
            value = self.device.controller.temperature
        else:
            if False:
                print("warning: %s update: not handled" % self.name)
            return
        if value != self.value:
            self.set_cached_value(value)
            self.device.notify_property_changed(self)




# -*- mode: python; python-indent-offset: 4; indent-tabs-mode: nil -*-
# SPDX-License-Indentifier: MPL-2.0

"""SenseHat adapter for Mozilla WebThings Gateway."""

import threading
import time

from gateway_addon import Device, Property

_POLL_INTERVAL = 1

_DEBUG = False

class SenseHatImuDevice(Device):
    """SenseHat IMU device type."""

    def __init__(self, adapter):
        """
        Initialize the object.

        adapter -- the Adapter managing this device
        """

        Device.__init__(self, adapter, 'sense-hat-imu')

        self._id = self.id = 'sense-hat-imu'
        self.adapter = adapter
        self.controller = adapter.controller
        self.controller.set_imu_config(True, True, True)

        self.name = 'SenseHatImu'
        self.description = 'Expose SenseHat IMU sensors'
        self.links = [
            {
                'rel': 'alternate',
                'mediaType': 'text/html',
                'href': adapter.URL
            }
        ]
        self._type = ['MultiLevelSensor']
        try:
            self.properties['yaw'] = SenseHatImuProperty(
                self,
                'yaw',
                {
                    '@type': 'LevelProperty',
                    'label': "Yaw",
                    'type': 'number',
                    'description': "Yaw Angle (North)",
                    'unit': 'º',
                    'minimum': -180,
                    'maximum': 180,
                    'readOnly': True
                },
                0)

            self.properties['pitch'] = SenseHatImuProperty(
                self,
                'pitch',
                {
                    '@type': 'LevelProperty',
                    'label': "Pitch",
                    'type': 'number',
                    'description': 'Pitch Angle',
                    'unit': 'º',
                    'minimum': -180,
                    'maximum': 180,
                    'readOnly': True
                },
                0)
            self.properties['roll'] = SenseHatImuProperty(
                self,
                'roll',
                {
                    '@type': 'LevelProperty',
                    'label': "Roll",
                    'type': 'number',
                    'description': "Roll Angle",
                    'unit': 'º',
                    'minimum': -180,
                    'maximum': 180,
                    'readOnly': True
                },
                0)

            self.properties['compass'] = SenseHatImuProperty(
                self,
                'compass',
                {
                    '@type': 'OnOffProperty',
                    'label': "Compass",
                    'type': 'boolean',
                    'readOnly': False
                },
                True)

            self.properties['gyro'] = SenseHatImuProperty(
                self,
                'gyro',
                {
                    '@type': 'OnOffProperty',
                    'label': "Gyro",
                    'type': 'boolean',
                    'readOnly': False
                },
                True)

            self.properties['accel'] = SenseHatImuProperty(
                self,
                'accel',
                {
                    '@type': 'OnOffProperty',
                    'label': "Accel",
                    'type': 'boolean',
                    'readOnly': False
                },
                True)

            thread = threading.Thread(target=self.poll)
            thread.daemon = True
            thread.start()

            self.pairing = True
            print("info: Adapter started")
        except Exception as ex:
            print("error: Adding properties: " + str(ex))

    def poll(self):
        """Poll the device for changes."""
        while True:
            time.sleep(_POLL_INTERVAL)
            try:
                for prop in self.properties.values():
                    prop.update()
            except Exception as ex:
                print("error: Polling properties: " + str(ex))
                print(prop.name)
                continue


class SenseHatImuProperty(Property):
    """ Webthing property for motion sensors"""

    def __init__(self, device, name, description, value):
        """ Initialize the object """
        Property.__init__(self, device, name, description)
        self.device = device
        self.title = name
        self.name = name
        self.description = description
        self.value = value
        self.set_cached_value(value)

    def update(self):
        """ read sensors values and notify """
        if self.name == 'pitch' or \
           self.name == 'roll' or \
           self.name == 'yaw':
            orientation = self.device.controller.orientation
            value = orientation[self.name] - 180.
            if _DEBUG:
                print("update: %s=%f" % (self.name, value))
        else:
            if _DEBUG:
                print("warning: %s update: not handled" % self.name)
            return
        if value != self.value:
            self.set_cached_value(value)
            self.device.notify_property_changed(self)


    def set_value(self, value):
        """ En/Dis/able sensors """
        if value != self.value:
            config = {'compass_enabled': self.device.properties['compass'].value,
                      'gyro_enabled': self.device.properties['gyro'].value,
                      'accel_enabled': self.device.properties['accel'].value}
            config[self.name + "_enabled"] = value
            print(config)
            self.device.controller.set_imu_config(**config)
            self.set_cached_value(value)
            self.device.notify_property_changed(self)



# -*- mode: python; python-indent-offset: 4; indent-tabs-mode: nil -*-
# SPDX-License-Indentifier: MPL-2.0

"""A setuptools based setup module."""

from codecs import open
from os import path
from setuptools import setup, find_packages


here = path.abspath(path.dirname(__file__))

# Get the long description from the README file.
with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

requirements = [
    'sense_hat==2.2.0'
]
URL = 'https://github.com/rzr/sense-hat-webthing'

setup(
    name='sense-hat-webthing',
    version='0.0.7',
    description='Sense Hat WebThing',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url=URL,
    author='Philippe Coval',
    author_email='rzr@users.sf.net',
    keywords='sensehat sense-hat RaspberryPi mozilla iot web thing webthing',
    packages=find_packages(),
    install_requires=requirements,
    classifiers=[
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Mozilla Public License 2.0 (MPL 2.0)',
        'Programming Language :: Python :: 3.7',
    ],
    license='MPL-2.0',
    project_urls={
        'Source': URL,
        'Tracker': URL + '/issues',
    },
    python_requires='>=3.7se, <4',
)



# -*- mode: python; python-indent-offset: 4; indent-tabs-mode: nil -*-
# SPDX-License-Indentifier: MPL-2.0

"""SenseHat adapter for Mozilla WebThings Gateway."""

from gateway_addon import Device, Property

_DEBUG = True

class SenseHatLightDevice(Device):
    """SenseHat device type."""

    def __init__(self, adapter):
        """
        Initialize the object.

        adapter -- the Adapter managing this device
        """
        Device.__init__(self, adapter, 'sense-hat-light')
        self._id = 'sense-hat-light'
        self.id = 'sense-hat-light'
        self.adapter = adapter
        self.controller = adapter.controller

        self.name = 'SenseHatLight'
        self.description = 'Expose SenseHat actuators'
        self.links = [
            {
                'rel': 'alternate',
                'mediaType': 'text/html',
                'href': adapter.URL
            }
        ]
        self._type = ['ColorControl', 'Light', 'OnOffSwitch']
        try:
            self.properties['character'] = SenseHatProperty(
                self,
                'character',
                {
                    '@type': 'StringProperty',
                    'label': "Character",
                    'type': 'string'
                },
                "")
            self.controller.show_letter("")

            self.properties['message'] = SenseHatProperty(
                self,
                "message",
                {
                    '@type': 'StringProperty',
                    'label': "Message",
                    'type': 'string'
                },
                "")
            self.controller.show_message("")

            self.properties['color'] = SenseHatProperty(
                self,
                "color",
                {
                    '@type': 'ColorProperty',
                    'label': "Color",
                    'type': 'string',
                    'readOnly': False
                },
                '#ffffff')
            self.controller.clear([0xFF, 0xFF, 0xFF])

            self.properties['on'] = SenseHatProperty(
                self,
                "on",
                {
                    '@type': 'OnOffProperty',
                    'label': "Switch",
                    'type': 'boolean',
                    'readOnly': False
                },
                False)
            self.controller.clear([0, 0, 0])

            self.properties['rotation'] = SenseHatProperty(
                self,
                "rotation",
                {
                    '@type': 'NumberProperty',
                    'label': "Rotation",
                    'type': 'integer',
                    'description': 'Rotation of LED matrix',
                    'unit': 'degrees',
                    'enum': [0, 90, 180, 270]
                },
                0)
            self.controller.set_rotation(0)

            self.properties['dim'] = SenseHatProperty(
                self,
                'dim',
                {
                    '@type': 'BooleanProperty',
                    'label': "Dim",
                    'type': 'boolean',
                    'description': 'Low light for dark environement',
                },
                False)
            self.controller.low_light = False

            self.pairing = True
            print("info: Adapter started")

        except Exception as ex:
            print("error: Adding properties: " + str(ex))


    @staticmethod
    def hex_to_rgb(text):
        """ Convert #RRBBGG to list of integer [0-255]"""
        color = [int(text[1:3], 0x10),
                 int(text[3:5], 0x10),
                 int(text[5:7], 0x10)]
        return color

    @staticmethod
    def invert_color(bg_color):
        """ Invert color from list of integers"""
        fg_color = [~ int(hex(bg_color[0]), 0x10) & 0xFF,
                    ~ int(hex(bg_color[1]), 0x10) & 0xFF,
                    ~ int(hex(bg_color[2]), 0x10) & 0xFF]
        return fg_color

    def show(self, **kargs):
        """ Refresh matrix light """
        args = {}
        for prop in self.properties.keys():
            if prop not in kargs:
                args[prop] = self.properties[prop].value
            else:
                args[prop] = kargs[prop]
        character = str(args['character'] if args['character'] else " ")[0:1]
        if args['on']:
            bg_color = SenseHatLightDevice.hex_to_rgb(str(args['color']))
            fg_color = SenseHatLightDevice.invert_color(bg_color)
        else:
            bg_color = [0, 0, 0]
            fg_color = SenseHatLightDevice.hex_to_rgb(str(args['color']))

        self.controller.show_letter(character, fg_color, bg_color)
        if 'message' in kargs.keys():
            self.controller.show_message(args['message'], 0.1, fg_color, bg_color)

class SenseHatProperty(Property):
    """ Matrix LCD parms"""

    def __init__(self, device, name, description, value):
        Property.__init__(self, device, name, description)
        self.device = device
        self.title = name
        self.name = name
        self.description = description
        self.value = value
        self.set_cached_value(value)

    def set_value(self, value):
        """ Handle properties changes """
        if _DEBUG:
            print("info: sense_hat." + self.name + " from " + str(self.value) + " to " + str(value))
        if value == self.value:
            return
        if self.name == 'rotation':
            if value != 0 and value != 90 and value != 180 and value != 270:
                print("warning: rotation must be 0, 90, 180 or 270")
                return
            if value != self.value:
                self.device.controller.set_rotation(value)

        elif self.name == 'dim':
            self.device.controller.low_light = value

        else:
            args = {}
            args[self.name] = value
            self.device.show(**args)

        self.set_cached_value(value)
        self.device.notify_property_changed(self)
